/*
 * $Id: RangeFinderTool.java,v 1.1.1.1 2004/12/14 00:25:39 norman Exp $
 *
 * Copyright (C) 2004 by Brockmann Consult (info@brockmann-consult.de)
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation. This program is distributed in the hope it will
 * be useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
package com.bc.beam.visat.rangeFinder;

import org.esa.beam.framework.datamodel.GeoCoding;
import org.esa.beam.framework.ui.ModalDialog;
import org.esa.beam.framework.ui.product.ProductSceneView;
import org.esa.beam.framework.ui.tool.AbstractTool;
import org.esa.beam.framework.ui.tool.ToolInputEvent;
import org.esa.beam.visat.VisatApp;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

/**
 * A tool representing the range finder.
 * @author Sabine Embacher
 */
class RangeFinderTool extends AbstractTool {

    public static final String TITLE = "Range Finder Tool";

    private final Point _currPoint;
    private Point _firstPoint;
    private int _numPoints;
    private Cursor _cursor;

    public RangeFinderTool() {
        _currPoint = new Point();
        _cursor = createRangeFinderCursor();
    }

    public Cursor getCursor() {
        return _cursor;
    }

    public void draw(Graphics2D g2d) {
        if (_numPoints == 0) {
            return;
        }

        final Stroke strokeOld = g2d.getStroke();
        g2d.setStroke(new BasicStroke(0.1f));
        final Color colorOld = g2d.getColor();
        g2d.setColor(Color.red);
        g2d.translate(0.5, 0.5);

        final int r = 3;
        final int r2 = r * 2;

        final Point p1 = _firstPoint;
        g2d.drawOval(p1.x - r, p1.y - r, r2, r2);
        g2d.drawLine(p1.x, p1.y - r2, p1.x, p1.y - r);
        g2d.drawLine(p1.x, p1.y + r2, p1.x, p1.y + r);
        g2d.drawLine(p1.x - r2, p1.y, p1.x - r, p1.y);
        g2d.drawLine(p1.x + r2, p1.y, p1.x + r, p1.y);

        final Point p2 = _currPoint;
        g2d.drawLine(p1.x, p1.y, p2.x, p2.y);

        g2d.translate(-0.5, -0.5);
        g2d.setStroke(strokeOld);
        g2d.setColor(colorOld);
    }

    public void mouseDragged(ToolInputEvent e) {
        super.mouseDragged(e);
        handleDragAndMove(e);
    }

    public void mouseMoved(ToolInputEvent e) {
        super.mouseMoved(e);
        handleDragAndMove(e);
    }

    public void mouseClicked(ToolInputEvent e) {
        super.mouseClicked(e);
        if (_numPoints == 0) {
            _firstPoint = e.getPixelPos();
            _currPoint.setLocation(e.getPixelPos());
            _numPoints++;
            repaint();
        } else {
            _numPoints = 0;

            ProductSceneView view = VisatApp.getApp().getSelectedProductSceneView();
            GeoCoding geoCoding = view.getProduct().getGeoCoding();

            final DistanceData dd = new DistanceData(geoCoding, _firstPoint, e.getPixelPos());

            final JButton detailsButton = new JButton("Details...");
            detailsButton.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    final Window parentWindow = SwingUtilities.getWindowAncestor(detailsButton);
                    ModalDialog detailsDialog = createDetailsDialog(parentWindow, dd);
                    detailsDialog.show();
                }
            });

            final JPanel buttonPane = new JPanel(new BorderLayout());
            buttonPane.add(detailsButton, BorderLayout.EAST);

            final JPanel messagePane = new JPanel(new BorderLayout(0, 6));
            messagePane.add(new JLabel("Distance: " + dd.distance + " +/- " + dd.distanceError + " km"));
            messagePane.add(buttonPane, BorderLayout.SOUTH);

            JOptionPane.showMessageDialog(VisatApp.getApp().getMainFrame(), messagePane, TITLE, JOptionPane.INFORMATION_MESSAGE);
            finish();
        }
    }

    private void handleDragAndMove(ToolInputEvent e) {
        if (_numPoints == 1) {
            _currPoint.setLocation(e.getPixelPos());
            repaint();
        }
    }

    private ModalDialog createDetailsDialog(final Window parentWindow, final DistanceData dd) {
        final String message =
                "Distance between points in pixels:\n" +
                "XH[" + dd.xH + "] to XN[" + dd.xN + "]: " + Math.abs(dd.xH - dd.xN) + "\n" +
                "YH[" + dd.yH + "] to YN[" + dd.yN + "]: " + Math.abs(dd.yH - dd.yN) + "\n" +
                "\n" +
                "LonH: " + dd.lonH + "   LatH: " + dd.latH + "\n" +
                "LonN: " + dd.lonN + "   LatN: " + dd.latN + "\n" +
                "\n" +
                "LamH: " + dd.lamH + "   PhiH: " + dd.phiH + "\n" +
                "LamN: " + dd.lamN + "   PhiN: " + dd.phiN + "\n" +
                "\n" +
                "Mean earth radius used: " + DistanceData.MEAN_EARTH_RADIUS_KM + " km" + "\n" +
                "\n" +
                "Distance: " + dd.distance + " +/- " + dd.distanceError + " km";           /*I18N*/

        final JScrollPane content = new JScrollPane(new JTextArea(message));
        content.setPreferredSize(new Dimension(300, 150));

        final ModalDialog detailsWindow = new ModalDialog(parentWindow,
                                                          TITLE + " - Details", /*I18N*/
                                                          ModalDialog.ID_OK,
                                                          null);
        detailsWindow.setContent(content);
        return detailsWindow;
    }

    private void repaint() {
        getDrawingEditor().repaintTool();
    }

    private Cursor createRangeFinderCursor() {
        Toolkit defaultToolkit = Toolkit.getDefaultToolkit();
        final String cursorName = "rangeFinder";
        ImageIcon icon = RangeFinderVPI.loadImageIcon("RangeFinderCursor.gif");

        // diese drei Zeilen sind Ntig, weil die Verschiedenen Platformen das bergebene
        // cursor image auf eine Andere Gre bringen, ohne jedoch dabei den Hotspot mit zu skalieren.
        Dimension bestCursorSize = defaultToolkit.getBestCursorSize(icon.getIconWidth(), icon.getIconHeight());
        Point hotSpot = new Point((7 * bestCursorSize.width) / icon.getIconWidth(),
                                  (7 * bestCursorSize.height) / icon.getIconHeight());

        return defaultToolkit.createCustomCursor(icon.getImage(), hotSpot, cursorName);
    }
}
